# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-16 09:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0004_auto_20180216_0935'),
    ]

    operations = [
        migrations.AlterField(
            model_name='modelchangelog',
            name='information',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelObject'),
        ),
    ]
